package com.davenicolette.atm;

import discrete.io.DioHandler;

public class AnsiStandardDispenser implements Dispenser {
	
	protected double cashAvailable;
	protected double pendingAmount;
	
	private DioHandler dio;
	
	void setDioHandler(DioHandler dio) {
		this.dio = dio;
	}

	private final int[][] denominations = new int[][] {
			{ DioHandler.TRAY5, 100 },
			{ DioHandler.TRAY4, 50 },
			{ DioHandler.TRAY3, 20 },
			{ DioHandler.TRAY2, 10 },
			{ DioHandler.TRAY1, 5 }
	};
	
	public double cashAvailable() {
		return cashAvailable;
	}

	public void dispenseCash() throws InvalidOperationException, DeviceFaultException {
		if ( pendingAmount <= 0.0 ) {
			throw new InvalidOperationException();
		}
		for ( int tray = 0 ; tray < denominations.length ; tray++ ) {
			int notes = ((int) pendingAmount) / denominations[tray][1];
			double amountDispensed = 0.0;
			while (notes > 0) {
				dio.put(denominations[tray][0]);
				if ( dio.get(DioHandler.FAULT, 5L) == 1 ) {
					throw new DeviceFaultException();
				}
				amountDispensed = denominations[tray][1];
				cashAvailable -= amountDispensed;
				pendingAmount -= amountDispensed;
				notes--;
			}
		}
	}

	public void requestCash(double amountRequested) throws AtmLowCashException {
		if ( amountRequested > cashAvailable ) {
			throw new AtmLowCashException();
		}
		pendingAmount = amountRequested;
	}

	public void reset() {
		pendingAmount = 0.0;
	}

}
