package com.davenicolette.atm;


public class Atm {
	
	private AccountLookup account;
	private Dispenser dispenser;
	private Logger logger;
	private DisplayManager displayManager;
	
	private AtmStatus status = AtmStatus.OUT_OF_SERVICE;
	
	void setStatus(AtmStatus status, long messageDelay) {
		this.status = status;
		displayManager.showMessage(status, messageDelay);
	}
	
	AtmStatus getStatus() {
		return status;
	}
	
	void setAccountLookup(AccountLookup account) {
		this.account = account;
	}
	
	void setDispenser(Dispenser dispenser) {
		this.dispenser = dispenser;
	}
	
	void setLogger(Logger logger) {
		this.logger = logger;
	}

	public void setDisplayManager(DisplayManager displayManager) {
		this.displayManager = displayManager;
	}
	
	public void withdrawCash(String accountNumber, double amount) {
		int transactionId = 0;
		try {
			transactionId = account.beginTransaction();
			dispenser.requestCash(amount);
			account.debit(accountNumber, amount); 
			dispenser.dispenseCash();
			account.commit(transactionId);
			logger.log("Cash withdrawal - Account " + accountNumber + ", amount " + amount);
			setStatus(AtmStatus.NEW_TRANSACTION, 0L);
		} catch (AtmLowCashException e) {
			dispenser.reset();
			logger.log("Cash withdrawal - Account " + accountNumber + ", ATM insufficient cash on hand; no debit");
			setStatus(AtmStatus.ATM_LOW_CASH, 0L);
			setStatus(AtmStatus.OUT_OF_SERVICE, 25000L);
		} catch (InsufficientFundsException e) {
			rollback(account, transactionId);
			dispenser.reset();
			logger.log("Cash withdrawal - Account " + accountNumber + ", Insufficient funds; no cash dispensed");
			setStatus(AtmStatus.INSUFFICIENT_FUNDS, 0L);
			setStatus(AtmStatus.NEW_TRANSACTION, 25000L);
		} catch (Exception e) {
			rollback(account, transactionId);
			dispenser.reset();
			logger.log("Cash withdrawal - Account " + accountNumber + ", Account in wrong status; no cash dispensed");
			setStatus(AtmStatus.CONTACT_BANK, 0L);
			setStatus(AtmStatus.NEW_TRANSACTION, 25000L);			
		}
	}
	
	/**
	 * Wrap exceptions on rollback.
	 */
	private void rollback(AccountLookup account, int transactionId) {
		try {
			account.rollback(transactionId);
		} catch (Exception e) {
			logger.log("Cash withdrawal - exception on rollback - account data may be inconsistent");
		}
	}
	
}
